            #GIT_CMT=33a5c626abf1f94b7104e2ef9a4a5881ca4507c3
# To disable particular handlers, remove the comment and space
#% global without_glfs -Dwith-glfs=FALSE
%global _without_rbd -Dwith-rbd=FALSE
#% global _without_qcow -Dwith-qcow=FALSE
%global _without_zbc -Dwith-zbc=FALSE
#% global _without_fbo -Dwith-fbo=FALSE
%global _without_tcmalloc -Dwith-tcmalloc=FALSE

Name:           tcmu-runner
License:        ASL 2.0 or LGPLv2+
Group:          System Environment/Daemons
Summary:        A daemon that handles the userspace side of the LIO TCM-User backstore
Version:        1.4.1
Release:    3.el7
Vendor:         Oracle America
URL:            https://github.com/open-iscsi/tcmu-runner
Source: tcmu-runner-1.4.1.tar.bz2

BuildRequires:  cmake make gcc
BuildRequires:  libnl3-devel glib2-devel zlib-devel kmod-devel

%if 0%{!?_without_glfs:1}
BuildRequires:  glusterfs-api-devel
Requires(pre):  glusterfs-api
%endif
%if 0%{!?_without_rbd:1}
BuildRequires:  librbd1-devel librados2-devel
Requires(pre):  librbd1 librados2
%endif
%if 0%{!?_without_tcmalloc:1}
BuildRequires:  gperftools-devel
Requires:       gperftools-libs
%endif

Requires(pre):  kmod zlib libnl3 glib2 logrotate rsyslog targetcli
Requires:       libtcmu = %{version}-%{release}

%description
A daemon that handles the userspace side of the LIO TCM-User backstore.

LIO is the SCSI target in the Linux kernel. It is entirely kernel code, and
allows exported SCSI logical units (LUNs) to be backed by regular files or
block devices. But, if we want to get fancier with the capabilities of the
device we're emulating, the kernel is not necessarily the right place. While
there are userspace libraries for compression, encryption, and clustered
storage solutions like Ceph or Gluster, these are not accessible from the
kernel.

The TCMU userspace-passthrough backstore allows a userspace process to handle
requests to a LUN. But since the kernel-user interface that TCMU provides
must be fast and flexible, it is complex enough that we'd like to avoid each
userspace handler having to write boilerplate code.

tcmu-runner handles the messy details of the TCMU interface -- UIO, netlink,
pthreads, and DBus -- and exports a more friendly C plugin module API. Modules
using this API are called "TCMU handlers". Handler authors can write code just
to handle the SCSI commands as desired, and can also link with whatever
userspace libraries they like.

%if 0%{!?_without_glfs:1}
%package handler-glfs
Summary:        Gluster handler for tcmu-runner
Requires:       %{name} = %{version}-%{release}

%description handler-glfs
A daemon that handles the complexity of the LIO kernel target's userspace
passthrough interface (TCMU). It presents a C plugin API for extension modules
that handle SCSI requests in ways not possible or suitable to be handled
by LIO's in-kernel backstores.

This package contains the Gluster handler.
%endif

%if 0%{!?_without_rbd:1}
%package handler-rbd
Summary:        Ceph RBD handler for tcmu-runner
Requires:       %{name} = %{version}-%{release}

%description handler-rbd
A daemon that handles the complexity of the LIO kernel target's userspace
passthrough interface (TCMU). It presents a C plugin API for extension modules
that handle SCSI requests in ways not possible or suitable to be handled
by LIO's in-kernel backstores.

This package contains the Ceph RBD handler.
%endif

%package -n libtcmu
Summary:       A library supporting LIO TCM-User backstores processing
Group:         Development/Libraries

%description -n libtcmu
libtcmu provides a library for processing SCSI commands exposed by the
LIO kernel target's TCM-User backend.

%package -n libtcmu-devel
Summary:        Development headers for libtcmu
Group:          Development/Libraries 
Requires:       %{name} = %{version}-%{release}
Requires:       libtcmu = %{version}-%{release}

%description -n libtcmu-devel
Development header(s) for developing against libtcmu.

%global debug_package %{nil}

%prep
%setup -q -n %{name}-%{version}%{?candidate:-%{candidate}}

%build
%cmake -DSUPPORT_SYSTEMD=ON \
%{?_without_glfs} \
%{?_without_rbd} \
%{?_without_zbc} \
%{?_without_qcow} \
%{?_without_fbo} \
%{?_without_tcmalloc} .
make %{?_smp_mflags}

%install
make install DESTDIR=%{buildroot}
%{__rm} -f %{buildroot}/etc/tcmu/tcmu.conf.old
%{__rm} -f %{buildroot}/etc/logrotate.d/tcmu-runner.bak/tcmu-runner

%post -n libtcmu -p /sbin/ldconfig

%postun -n libtcmu -p /sbin/ldconfig

%files
%{_bindir}/tcmu-runner
%dir %{_sysconfdir}/dbus-1/
%dir %{_sysconfdir}/dbus-1/system.d
%config %{_sysconfdir}/dbus-1/system.d/tcmu-runner.conf
%dir %{_datadir}/dbus-1/
%dir %{_datadir}/dbus-1/system-services/
%{_datadir}/dbus-1/system-services/org.kernel.TCMUService1.service
%{_unitdir}/tcmu-runner.service
%dir %{_libdir}/tcmu-runner/
%{_libdir}/tcmu-runner/*.so
%{_mandir}/man8/*
%doc README.md LICENSE.LGPLv2.1 LICENSE.Apache2
%dir %{_sysconfdir}/tcmu/
%config %{_sysconfdir}/tcmu/tcmu.conf
%config(noreplace) %{_sysconfdir}/logrotate.d/tcmu-runner
%ghost %attr(0644,-,-) %{_sysconfdir}/tcmu/tcmu.conf.old
%ghost %attr(0644,-,-) %{_sysconfdir}/logrotate.d/tcmu-runner.bak/tcmu-runner


%if 0%{!?_without_glfs:1}
%files handler-glfs
%{_libdir}/tcmu-runner/handler_glfs.so
%endif

%if 0%{!?_without_rbd:1}
%files handler-rbd
%{_libdir}/tcmu-runner/handler_rbd.so
%endif

%files -n libtcmu
%{_libdir}/libtcmu*.so.*

%files -n libtcmu-devel
%{_includedir}/libtcmu.h
%{_includedir}/libtcmu_common.h
%{_libdir}/libtcmu*.so

%changelog
* Fri Jan 15 2021 Tiger Yang <tiger.yang@oracle.com> - 1.4.1-3.el7
- build 1.4.1-3 (Tiger Yang)  
- tcmur: error if both src/dst_dev are unset after CSCD parsing (David Disseldorp)  [Orabug: 32368811] 
- tcmur: don't assume two XCOPY CSCDs (David Disseldorp)  [Orabug: 32368811] 
- tcmur: fail XCOPY requests with inline data (David Disseldorp)  [Orabug: 32368811] 
- tcmur: fail cross-device XCOPY requests (David Disseldorp)  [Orabug: 32368811]  {CVE-2020-28374}
- build 1.4.1-2 for ol8 (Tiger Yang)  

* Wed Jun 26 2019 Tiger Yang <tiger.yang@oracle.com> - 1.4.1-2.el7
- merge upstream and build 1.4.1-2

* Sun Jun 23 2019 Tiger Yang <tiger.yang@oracle.com> - 1.4.1-2
- conflict fixed
- Merge remote-tracking branch 'open-iscsi/master' into oracle-gluster-6
- Merge pull request #560 from pengliang233/out_of_bounds
- Merge pull request #562 from dillaman/wip-tcmalloc-opts
- cmake: set '-fno-builtin-XYZ' options when tcmalloc is used
- Merge pull request #558 from lxbsz/spec
- configfs: fix index out of bounds
- Merge pull request #559 from ddiss/libtcmu_sovers
- cmake: add a minor .so version number to libtcmu
- spec: track backuped files
- spec: update the spec file to fix the build error
- spec: pass the tcmalloc option to cmake command
- spec: switch to use rpm conditional switches
- Merge pull request #557 from dillaman/wip-install-dep
- extra: add gperftools-devel to install_dep script
- Merge pull request #555 from dillaman/wip-tcmalloc
- Optionally link against tcmalloc for improved small IO performance
- Merge pull request #551 from lxbsz/header
- libtcmu_time: void header file include muti-times
- Merge pull request #550 from lxbsz/lock
- Merge pull request #549 from lxbsz/glfs
- glfs: update the glfs logdir
- README: update to add uninstall
- lockfile: switch to use /run/tcmu.lock
- Merge pull request #547 from lxbsz/cmake
- logrotate: fix duplicate log entry error
- Validate input for tcmu_log_dir_check()
- Fix parameter for tcmu_setup_log()
- Merge pull request #545 from geordieintheshellcode/query-oooc-support
- Follow coding style, move leading brace
- Add method to query whether OOOC is supported
- Merge pull request #541 from lxbsz/glfs
- glfs: fix build error for GFAPI version check
- Allow some commands to run while taking lock
- Drop in_lock_thread from __tcmu_reopen_dev
- Fix pthread error handling
- Check if device is opened before calling handler
- file_zbc: optionally build zbc handler
- file_zbc: fixed compile error under ppc64le
- Merge pull request #539 from lxbsz/glfs
- glfs: add gfapi version check
- Merge pull request #532 from bgly/fix_unmap
- Merge pull request #538 from pengliang233/fix_typo
- Fix unmap to handle refcount being 0
- fix typo in code
- Merge pull request #535 from ktdreyer/rpm-rm-clean
- Merge pull request #536 from ktdreyer/rpm-rm-defattr
- rpm: remove %defattr
- rpm: remove %clean section
- Merge pull request #531 from ktdreyer/rpm-license-tag
- rpm: use valid short name licenses in License field
- Merge pull request #530 from pengliang233/clean_up
- configfs: clean up the code
- Merge pull request #529 from pengliang233/pthread_ret
- Fix pthread error ret
- Merge pull request #526 from pengliang233/fix_unlock
- rbd: fix the error ret in tcmu_rbd_unlock
- Merge pull request #528 from lmh10144360/droparg
- Merge pull request #522 from pengliang233/set_genl_features
- Drop sense arg from tcmu_explicit_transition
- libtcmu: setup_netlink maybe failure but can't sensed
- Merge pull request #525 from mikechristie/reopen-fixes
- Drop is_sync arg from tcmu_acquire_dev_lock

* Mon Apr 29 2019 Tiger Yang <tiger.yang@oracle.com> - 1.4.1-1
- tcmu-runner: bump version to 1.4.1
- Allow some commands to run while taking lock
- Drop in_lock_thread from __tcmu_reopen_dev
- Fix pthread error handling
- Check if device is opened before calling handler
- file_zbc: optionally build zbc handler
- file_zbc: fixed compile error under ppc64le

* Tue Apr 16 2019 Tiger Yang <tiger.yang@oracle.com> - 1.4.0-1
- tcmu-runner: bump version to 1.4.0
- Disable explicit alua support.
- Merge pull request #482 from lxbsz/dyn3
- logger: notification on changing the options for dynamic reloading
- logger: update .gitignore for logrotate.conf_install.cmake
- logger: fix the return value
- Merge pull request #480 from mikechristie/log-fixes
- Sync sig handler naming.
- Add logrotate support
- Move starting log message.
- Allow log file restart from SIGHUP
- Simplify log outputs
- Simplify tcmu_logbuf init
- make config watcher optional
- log/config: limit callers that can set the log dir
- logger: return errno instead of bool
- misc logcleanup
- Fix logger thread race
- Log rb cleanup
- log: move tcmu_logbuf->finish_initialize into logbuf lock/unlock
- tcmu-runner: add Starting log info
- Merge pull request #477 from mikechristie/lock-bounce-fixes
- Merge pull request #469 from mikechristie/io-size-fixes
- runner/rbd: do not set lock metadata if not needed
- runner: don't drop iscsi connection on lock fence errors
- runner: print lio name and not the uio name
- libtcmu: fix max unmap definition
- rbd: increase xcopy IO size
- tcmu-runner: set default internal xcopy limits based on max xfer len
- libtcmu: add helpers for xcopy read/writes
- runner/rbd: pass unmap segments directly to handler
- rbd: set unmap limits based on object size
- runner: allow handler to override max unmap len limit
- runner: allow handlers to override default unmap limits
- Change alua transition in process message to a debug
- Add newline to alua lock messages.
- rbd: fix llu vs PRIu64 use for len/off
- Merge pull request #464 from DavidButterfield/master
- Merge pull request #472 from mikechristie/fix-infin-break-lock
- runner: reopen dev right away if we know we had the lock and lost it
- runner: fix double log message
- runner: fix double close/free
- Merge pull request #471 from mikechristie/fix-implicit-lock-state
- Do not return alua state transition
- runner/rbd: update lock state on RTPG/INQUIRY
- format string as ("%s", buf) instead of (buf)
- add checking of logging formats vs. arguments
- Merge pull request #467 from tangwenji/fix_max_seg_and_tgt_desc_num
- Merge pull request #466 from tangwenji/fix_cscd_min_len
- Merge pull request #465 from tangwenji/fix_sa_xcopy
- libtcmu: add additonal sense code for xcopy
- tcmur: fix the minimum of cscd descriptors list length
- tcmur: differentiate various sa in the xcopy command
- Merge pull request #459 from lxbsz/fix_code_style
- tcmur: fix the max length of caw command
- Merge pull request #462 from tangwenji/fix_sense_valid
- Merge pull request #463 from damien-lemoal/master
- file_zbc: Fixed write error checking
- libtcmu:fix valid field in sense data
- tcmu-runner: fix the code style
- Merge pull request #458 from tangwenji/fix_caw_zero_length
- tcmur:fix number of logical blocks is zero in caw command
- Fix compilation error due to iovec definition duplication
- Merge pull request #456 from pkalever/loglevel
- Merge pull request #457 from tangwenji/fix_formart_character
- tcmur:fix format character on handle_writesame_check
- glfs: set loglevel to INFO
- Merge pull request #455 from tangwenji/fix_recovery_del_node
- Merge pull request #453 from tangwenji/fix_free_io_watch
- Merge pull request #454 from tangwenji/fix_acquire_dev_lock
- target:delete device node from devs before reopen device
- tcmur:fix the initial value of the return var
- main:remove source and decrements the reference count of the GIOChannel
- Merge pull request #451 from dillaman/wip-rbd-device-classes
- rbd: detect pools associated to NVMe device class
- Merge pull request #409 from lxbsz/rpm_error
- spec: force to install tcmu.conf in buildroot
- Merge pull request #450 from lxbsz/spec1
- spec: fix rpm package build error
- Merge pull request #448 from Gangbiao/lgb-devel
- To enhance the reuseability of code, move function 'tcmu_cancel_thread' from 'main.c' to 'api.c'
- tcmu-runner: fix bug in HA about re-acquire lock
- Fix compilation errors
- Merge pull request #402 from lxbsz/reset_netlink
- Merge pull request #446 from DavidButterfield/master
- glfs: add discard support
- improve tcmu-runner behavior under valgrind
- Improve generality of tcmu_dev_{err,warn,...}
- tcmu-runner: add reset netlink support
- configfs: add CFGFS_TARGET_MOD macro
- Merge pull request #443 from damien-lemoal/master
- file_zbc: correctly handle iovec seek
- Merge pull request #437 from lxbsz/glfs1
- glfs: allow the backend file size not aligns to the block_size
- Fix tarball release location
- Merge pull request #433 from lxbsz/cmakelist
- CMakeList: fix missing file for static library
- Merge pull request #430 from lxbsz/main_syms
- main-syms: remove tcmu_cancel_thread
- Merge pull request #429 from lxbsz/config_fix1
- Merge pull request #428 from lxbsz/conf_fix
- config: fix parse error when there has no space after '='
- config: make the struct data invisible
- config: fix the log message and clean up the code
- Merge pull request #425 from mikechristie/relicense
- Merge pull request #368 from lxbsz/logdir_dyn
- log: do not insert the CDB debug logs to ringbuffer
- config: add the logdir option dynamic reloading support
- config: fix memory leak of cfg->log_dir_path
- config: reset to boolean default values after the options being set
- config: reset to str default values after the options being set
- Merge pull request #424 from lxbsz/lock
- alua: allow handlers missing lock routine
- Change license to Apache 2 or LGPLv2.1
- Add LGPL license file
- Rename license file
- Merge pull request #418 from pkalever/lockfile
- runner: don't allow multiple instances of daemon to run
- Merge pull request #415 from tjakob/master
- Merge pull request #417 from lxbsz/alua_log_fix1
- alua: reduce the debug logs
- Merge pull request #403 from lxbsz/task_full
- api: list all the cdb data for unsupported SCSI commands
- rbd: new option for specifying the id
- tcmu-runner: bump version to 1.4.0-rc1
- Merge pull request #413 from mikechristie/return-codes
- update docs for TCMU_STS codes
- cleanup unused sense buf uses
- Move SCSI sense defs to zbc
- Cleanup tcmulib_command_complete
- Drop info arg from tcmu_set_sense_data
- Convert ASC_NOT_READY_FORMAT_IN_PROGRESS
- Convert simple fbo cases to TCMU_STS
- Convert rbd alua/lock to use TCMU_STS
- Convert alua sense/status
- Convert simple sense/status cases
- Convert ASC_MISCOMPARE_DURING_VERIFY_OPERATION to TCMU_STS_MISCOMPARE
- Convert ASC_INTERNAL_TARGET_FAILURE
- Convert ASC_LBA_OUT_OF_RANGE to TCMU_STS_RANGE
- Convert SAM_STAT_TASK_SET_FULL to TCMU_STS_NO_RESOURCE
- Convert SAM_STAT_GOOD to TCMU_STS_OK
- Convert TCMU_ASYNC_HANDLED to TCMU_STS_ASYNC_HANDLED
- Convert TCMU_NOT_HANDLED to TCMU_STS_NOT_HANDLED
- Sync up error codes
- Merge pull request #407 from mikechristie/explicit-fo
- rbd/runner: remove get_lock_state
- rbd: get/set lock tag
- runner: get/set remote alua group info
- add back explicit failover support
- runner: separate connection errros from fence ones
- runner: return lock state after update
- runner: add new lock states
- runner: add helper to set alua ints
- runner: allow tcmu_acquire_dev_lock to be called from cmd handler
- runner: make reopen retries configurable
- Merge pull request #405 from letterwuyu/master
- Merge pull request #408 from lxbsz/gitignore1
- fix the memory cross boundary bug in tcmu_rbd_detect_device_class
- gitignore: ignore extra/rpmbuild
- Merge pull request #406 from baihuahua/dev
- libtcmu: drop duplicated ";"
- Merge pull request #401 from pkalever/retvalue
- tcmur: fix ret value in tcmur_dev_update_size
- Merge pull request #399 from tangwenji/fix_reopen
- fix the condition of sleep in _tcmu_reopen_dev
- Merge pull request #398 from lxbsz/config_fix
- config: reset to default values after the options being set
- Merge pull request #392 from dillaman/wip-batch-wake-ups
- tcmur: batch kernel wake up notifications
- Merge pull request #391 from lxbsz/log1
- tcmu-runner: improve the logging always to get daemon's exit state
- Merge pull request #382 from lxbsz/glfs_resize
- Merge pull request #389 from tangwenji/fix_rbd_parse_cfg
- rbd:fix parse cfgstring optional options
- Merge pull request #388 from pengliang233/fix-goto
- Merge pull request #386 from tangwenji/fix_free_manager
- Merge pull request #385 from tangwenji/fix_rbd_cfg_optional
- main: fix error goto
- tcmu-runner:fix free manager
- rbd:Parsing option 'conf=' error
- glfs: reopen on connection lost
- libtcmu: Add tcmu flags and size setting helpers support
- Merge pull request #383 from pengliang233/fix-ret
- conifg: fix error ret
- rbd: fix coding style and compilation error
- Merge pull request #365 from umuzhaohui/wip-fix-1
- tcmu-runner: fix bug in EXTENDED_COPY command
- tcmu-runner: fix bug in COMPARE_AND_WRITE command
- Merge pull request #378 from lxbsz/reset_ring
- Merge pull request #381 from lxbsz/fix_members
- libtcmu: add tcmu_is_ring_reset_support support
- configfs: remove null chars in the string readed from the sys files
- Merge pull request #375 from dmitry-fomichev/master
- Set correct sense upon getting write to FULL zone
- Add installation action for file_zbc handler
- zbc: fix maximum number of open zones
- zbc: fix write emulation
- Merge pull request #376 from draghuram/doc
- Fixed build procedure in README.
- Merge pull request #373 from lxbsz/fix_log_missing
- Merge pull request #372 from pkalever/size_checks
- logging: improve log details
- log: bypass the log ring buffer for file and std outputs
- glfs: remove size checks in handler
- Merge pull request #370 from ddiss/factory_pkg_1.3
- build: drop versionless libtcmu.so symlink
- api: avoid compiler -Werror=return-type failures
- Merge pull request #367 from lxbsz/extra1
- extra: Add make_runnerrpms.sh support
- Merge pull request #355 from zhuozh/fix_lock_treshing
- Merge pull request #357 from lxbsz/update_glfs1
- glfs: update the glfs config descritions and add usage of creation
- handler: do not perform implicit transition if command not supported

* Tue Jan 22 2019 Tiger Yang <tiger.yang@oracle.com> - 1.3.0-2
- rebuild for oracle-gluster-5

* Mon Aug 20 2018 Tiger Yang <tiger.yang@oracle.com> - 1.3.0-1
- Initial revision for oracle-gluster-4.1
- tcmu-runner: bump version to 1.3.0

